
#include "defines.h"
#include <inttypes.h>
#include <util/delay.h>
#include "i2csoft.h"
#include "mcp23017.h"
#include "serialio.h"
#define IN_COUNTER_CTL
#include "counter_ctl.h"


/* 
   ----------------------------------------------------------------------------
   GLOBALS
   ----------------------------------------------------------------------------
*/
static uint8_t flagbyte;
static uint8_t iobyte;
uint32_t unitx_count, unity_count, unitz_count;

/* 
   ----------------------------------------------------------------------------
   COUNTER BOARD EXPANDER CONNECTIONS
   ----------------------------------------------------------------------------
*/
/* IO Expander pin definitions:
 * EXPAN_1
 * GPIO A0  74LV8154 data bus Y0
 * GPIO A1  74LV8154 data bus Y1
 * GPIO A2  74LV8154 data bus Y2
 * GPIO A3  74LV8154 data bus Y3
 * GPIO A4  74LV8154 data bus Y4
 * GPIO A5  74LV8154 data bus Y5
 * GPIO A6  74LV8154 data bus Y6
 * GPIO A7  74LV8154 data bus Y7
 *
 * GPIO B0  INPUT - PPS(SYNC) pulse
 * GPIO B1  OUTPUT - RESET
 * GPIO B2  INPUT - HALT(RDY)
 * GPIO B3  OUTPUT - RCLK ('8154)
 * GPIO B4  OUTPUT - GATE SEL 1 SEC
 * GPIO B5  OUTPUT - GATE SEL 10 SEC
 * GPIO B6  OUTPUT - GATE SEL 100 SEC
 * GPIO B7  INPUT - UNUSED
 *
 * EXPAN_2
 * GPIO A0  UNIT X GAL
 * GPIO A1  UNIT X GAU
 * GPIO A2  UNIT X GBL
 * GPIO A3  UNIT X GBU
 * GPIO A4  UNIT Y GAL
 * GPIO A5  UNIT Y GAU
 * GPIO A6  UNIT Y GBL
 * GPIO A7  UNIT Y GBU
 *
 * GPIO B0  UNIT Z GAL
 * GPIO B1  UNIT Z GAU
 * GPIO B2  UNIT Z GBL
 * GPIO B3  UNIT Z GBU
 * GPIO B4  OUTPUT - UNUSED
 * GPIO B5  OUTPUT - UNUSED
 * GPIO B6  OUTPUT - UNUSED
 * GPIO B7  OUTPUT - UNUSED
 */


/* 
   ----------------------------------------------------------------------------
   COUNTER BOARD CONTROL FUNCTIONS
   ----------------------------------------------------------------------------
*/

/*----------------------------------------------------------------------------
 * FUNCTION: counter_init
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_init( void )
{
  /* set up EXPAN_1, GPIO A as Y bus input */
  mcp_write_byte(EXPAN_1, MCP_IODIRA, 0xFF); /* set all GPIOA as input */
  mcp_write_byte(EXPAN_1, MCP_GPPUA,  0xFF); /* pull ups ON for Y bus */

  /* set up EXPAN_1, GPIO B as control port */
  mcp_write_byte(EXPAN_1, MCP_IODIRB, 0b10000101); /* see map above */
  mcp_write_byte(EXPAN_1, MCP_GPPUB,  0x00);  /* pull ups OFF for control */
  mcp_write_byte(EXPAN_1, MCP_GPIOB,  0b00000000); /* lower RESET and RCLK */

  counter_gate_select( GATE_1SEC ); /* set gate to 1 sec */

  /* set up EXPAN_2, GPIO A as byte select for channel X and Y */
  mcp_write_byte(EXPAN_2, MCP_IODIRA, 0); /* all outputs */
  counter_byte_select( UNITX, BYTE_NONE );
  counter_byte_select( UNITY, BYTE_NONE );

  /* set up EXPAN_2, GPIO B as byte select for channel Z */
  mcp_write_byte(EXPAN_2, MCP_IODIRB, 0); /* all outputs */
  counter_byte_select( UNITZ, BYTE_NONE );
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_gate_select
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_gate_select( uint8_t gate_value )
{
  switch(gate_value)
  {
    case GATE_1SEC:
      mcp_write_byte( EXPAN_1, MCP_GPIOB, 0b00010000 );
      break;
    case GATE_10SEC:
      mcp_write_byte( EXPAN_1, MCP_GPIOB, 0b00100000 );
      break;
    case GATE_100SEC:
      mcp_write_byte( EXPAN_1, MCP_GPIOB, 0b01000000 );
      break;
  }
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_byte_select
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_byte_select( uint8_t unit, uint8_t byte_pos )
{
  /* make sure that only one counter bit is selected by de-selecting */
  /* all before selecting specific bit */
  mcp_write_byte( EXPAN_2, MCP_GPIOA, 0b11111111 );
  mcp_write_byte( EXPAN_2, MCP_GPIOB, 0b11111111 );

  switch( unit )
  {
    case UNITX:
      /* select X bit */
      mcp_write_byte( EXPAN_2, MCP_GPIOA, byte_pos );
      break;
    case UNITY:
      /* select Y bit */
      mcp_write_byte( EXPAN_2, MCP_GPIOA, (byte_pos<<4) | 0b00001111 );
      break;
    case UNITZ:
      /* select Z bit */
      mcp_write_byte( EXPAN_2, MCP_GPIOB, byte_pos );
      break;
  }
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_byte_select
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_do_count( void )
{
  /* wait for PPS(SYNC) to go high - looking for PPS pulse*/
  flagbyte=0;
  while( !( 0b00000001 & flagbyte ) )
  {
     mcp_read_byte( EXPAN_1, MCP_GPIOB, &flagbyte );
     _delay_loop_2( 10 ); /* wait for 40 cpu cycles */
  }

  /* now, wait for PPS(SYNC) to go low  */
  while( ( 0b00000001 & flagbyte ) )
  {
     mcp_read_byte( EXPAN_1, MCP_GPIOB, &flagbyte );
     _delay_loop_2( 10 ); /* wait for 40 cpu cycles */
  }

  /* PPS line just went low - send RESET */  
  counter_reset();
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_reset
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_reset( void )
{
  /* pulse RESET high for 1 ms */
  mcp_read_byte( EXPAN_1, MCP_GPIOB, &iobyte );
  iobyte |= 0b00000010;
  mcp_write_byte( EXPAN_1, MCP_GPIOB, iobyte );
  _delay_ms(1);
  iobyte &= 0b11111101;
  mcp_write_byte( EXPAN_1, MCP_GPIOB, iobyte );
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_reset
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_rclk( void )
{
  /* pulse RESET high for 1 ms */
  mcp_read_byte( EXPAN_1, MCP_GPIOB, &iobyte );
  iobyte |= 0b00001000;
  mcp_write_byte( EXPAN_1, MCP_GPIOB, iobyte );
  _delay_ms(1);
  iobyte &= 0b11110111;
  mcp_write_byte( EXPAN_1, MCP_GPIOB, iobyte );
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_ready
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
uint8_t counter_ready( void )
{
  uint8_t flagbyte;

  mcp_read_byte( EXPAN_1, MCP_GPIOB, &flagbyte );

  return (flagbyte & 0b00000100);
}


/*----------------------------------------------------------------------------
 * FUNCTION: counter_read
 * PARAMS  : 
 * RETURNS : 
 * CALLS   : 
 * DESCR   : 
 *           
 * --------------------------------------------------------------------------*/
void counter_read( void )
{
  /* clock the counter data into the output registers on the 748154s */
  counter_rclk();

  /* pull each counter byte value, with a 1ms delay in between */

  /* unit x */
  unitx_count=0L;
  counter_byte_select( UNITX, BYTE_GAL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
    unitx_count |= ((uint32_t)iobyte);
  counter_byte_select( UNITX, BYTE_GAU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
    unitx_count |= (((uint32_t)iobyte) << 8);
  counter_byte_select( UNITX, BYTE_GBL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
    unitx_count |= (((uint32_t)iobyte) << 16);
  counter_byte_select( UNITX, BYTE_GBU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
    unitx_count |= (((uint32_t)iobyte) << 24);

  /* unit y */
  unity_count=0L;
  counter_byte_select( UNITY, BYTE_GAL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unity_count |= ((uint32_t)iobyte);
  counter_byte_select( UNITY, BYTE_GAU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unity_count |= (((uint32_t)iobyte) << 8);
  counter_byte_select( UNITY, BYTE_GBL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unity_count |= (((uint32_t)iobyte) << 16);
  counter_byte_select( UNITY, BYTE_GBU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unity_count |= (((uint32_t)iobyte) << 24);


  /* unit z */
  unitz_count=0L;
  counter_byte_select( UNITZ, BYTE_GAL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unitz_count |= ((uint32_t)iobyte);
  counter_byte_select( UNITZ, BYTE_GAU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unitz_count |= (((uint32_t)iobyte) << 8);
  counter_byte_select( UNITZ, BYTE_GBL );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unitz_count |= (((uint32_t)iobyte) << 16);
  counter_byte_select( UNITZ, BYTE_GBU );
  /*_delay_ms(1);*/
  mcp_read_byte( EXPAN_1, MCP_GPIOA, &iobyte );
  unitz_count |= (((uint32_t)iobyte) << 24);
}
