/**********************************************************

Software I2C Library for AVR Devices.

Copyright 2008-2012
eXtreme Electronics, India
www.eXtremeElectronics.co.in
**********************************************************/

#include "defines.h"
#include <avr/io.h>
#include <util/delay.h>
#include "i2csoft.h"

/*_delay_loop_2(3)*/ /* was 5 each 20 works */

/*
 * Speed    Delay Loop Time     Equiv I2C Bus Speed
 * 16MHz    5                   800KHz
 * 16MHz    20                  400KHz
 * 16MHz    40                  100KHz
 * 16MHz    80                   50KHz
 */

#define Q_DEL _delay_loop_2(delay_value) 
#define H_DEL _delay_loop_2(delay_value)

static uint16_t delay_value = 40; /* default to 100kHz I2C bus */

/*
  Description:
  Initializes the Soft I2C Engine.
  Must be called before using any other lib functions.
  
  Arguments:
  NONE
  
  Returns:
  Nothing
*/
void i2c_init()
{
  delay_value = 40; /* default to 100kHz I2C bus */

  SDAPORT&=(1<<SDA);
  SCLPORT&=(1<<SCL);
  
  I2C_SDA_HIGH;	
  I2C_SCL_HIGH;			
}


/*
  Description:
  Set the speed of the software-driver I2C bus, based on processor
  speed.

  Arguments:
  Bus speed in KHz

  Returns:
  None
*/
void i2c_set_speed( uint32_t bus_speed )
{
  if( bus_speed == 0 )
    return;

  delay_value = ((uint16_t)( F_CPU / bus_speed )) / 4;
}


/*
  Description:
  Generates a START(S) condition on the bus.
  NOTE: Can also be used for generating repeat start(Sr)
  condition too.
  
  Arguments:
  NONE
  
  Returns:
  Nothing
*/
void i2c_start()
{
  I2C_SCL_HIGH;
  H_DEL;
  
  I2C_SDA_LOW;	
  H_DEL;  	
}


/*
  Description:
  Generates a STOP(P) condition on the bus.
  NOTE: Can also be used for generating repeat start
  condition too.
  
  Arguments:
  NONE
  
  Returns:
  Nothing
*/
void i2c_stop()
{
  I2C_SDA_LOW;
  H_DEL;
  I2C_SCL_HIGH;
  Q_DEL;
  I2C_SDA_HIGH;
  H_DEL;
}


/*
  Description:
  Sends a Byte to the slave.
  
  Arguments:
  8 bit date to send to the slave.
  
  Returns:
  non zero if slave acknowledge the data receipt.
  zero other wise.
*/
uint8_t i2c_write_byte(uint8_t data)
{
  uint8_t i;
  
  for(i=0;i<8;i++)
    {
      I2C_SCL_LOW;
      Q_DEL;
      
      if(data & 0x80)
	I2C_SDA_HIGH;
      else
	I2C_SDA_LOW;	
      
      H_DEL;
      
      I2C_SCL_HIGH;
      H_DEL;
      
      while((SCLPIN & (1<<SCL))==0);
      
      data=data<<1;
    }
  
  //The 9th clock (ACK Phase)
  I2C_SCL_LOW;
  Q_DEL;
  
  I2C_SDA_HIGH;		
  H_DEL;
  
  I2C_SCL_HIGH;
  H_DEL;	
  
  uint8_t ack=!(SDAPIN & (1<<SDA));
  
  I2C_SCL_LOW;
  H_DEL;
  
  return ack;
}


/*
  Description:
  Reads a byte from slave.
  
  Arguments:
  1 if you want to acknowledge the receipt to slave.
  0 if you don’t want to acknowledge the receipt to slave.
  
  Returns:
  The 8 bit data read from the slave.
*/
uint8_t i2c_read_byte(uint8_t ack)
{
  uint8_t data=0x00;
  uint8_t i;
  
  for(i=0;i<8;i++)
    {
      
      I2C_SCL_LOW;
      H_DEL;
      I2C_SCL_HIGH;
      H_DEL;
      
      while((SCLPIN & (1<<SCL))==0);
      
      if(SDAPIN &(1<<SDA))
	data|=(0x80>>i);
      
    }
  
  I2C_SCL_LOW;
  Q_DEL;						//Soft_I2C_Put_Ack
  
  if(ack)
    {
      I2C_SDA_LOW;	
    }
  else
    {
      I2C_SDA_HIGH;
    }	
  H_DEL;
  
  I2C_SCL_HIGH;
  H_DEL;
  
  I2C_SCL_LOW;
  H_DEL;
  
  return data; 
}















