/* Copyright (c) 2015-2017, Mike Smith
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * o  Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * o  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * o  Neither the name of the <ORGANIZATION> nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "defines.h"
#include <inttypes.h>
#include <util/delay.h>
#include "i2csoft.h"
#include "mcp23017.h"

/* ----------------------------------------------------------------------------
   I2C MCP23017 IO EXPANDER FUNCTIONS
   ----------------------------------------------------------------------------
*/

/*
 * MCP23017 Control Byte:
 *
 * b7..b4:  0100
 * b3..b1:  Address ( 001 ) 
 * b0:      R/W   Read = 1, Write = 0
 *
 * for now: 0100 000 0 for write  0x40
 *          0100 000 1 for read   0x41
 */

#define MCP_CTL_WRITE  0x40
#define MCP_CTL_READ   0x41

/* used to create read and write ctrl bytes with chip address */
static uint8_t ctl_write, ctl_read;

/*----------------------------------------------------------------------------
 * FUNCTION: mcp_write_byte
 * PARAMS  : 3 bit chip address, 8 bit register address, 8 bit data to write
 * RETURNS : MCP_IO_ERROR on failure, MCP_SUCCESS on success
 * CALLS   : i2c_start(), i2c_write_byte(), i2c_stop(), _delay_ms()
 * DESCR   : Write a byte 'data' to register address 'regaddr' to MCP23017 
 *           attached to I2C bus with chip address 'chipaddr'.
 * --------------------------------------------------------------------------*/
uint8_t mcp_write_byte(uint8_t chipaddr,uint8_t regaddr,uint8_t data)
{
  /* put chip address into control bytes */
  chipaddr = chipaddr & 0b00000111;
  if( chipaddr == EEPROM_ADDR )
    return MCP_IO_ERROR; /* do not use this module to talk to eeprom */
  chipaddr = chipaddr << 1; /* move address to b3/b2/b1 */
  ctl_write = MCP_CTL_WRITE | chipaddr;  /* put address into ctl bytes */
  ctl_read  = MCP_CTL_READ  | chipaddr;

  i2c_start();
  
  /* CTL Byte */
  if(!i2c_write_byte(ctl_write))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  /* Address */
  if(!i2c_write_byte(regaddr))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  /* Data */
  if(!i2c_write_byte(data))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  i2c_stop();
  
  _delay_ms(5);
  
  return MCP_SUCCESS;
}


/*----------------------------------------------------------------------------
 * FUNCTION: mcp_read_byte
 * PARAMS  : 3 bit chip address, 8 bit register address, pointer to 8 bit 
 *           data location for value read
 * RETURNS : MCP_IO_ERROR on failure, MCP_SUCCESS on success
 * CALLS   : i2c_start(), i2c_write_byte(), i2c_stop(), i2c_read_byte()
 * DESCR   : Read a byte 'data' from register address 'regaddr' from MCP23017 
 *           attached to I2C bus at chip address 'chipaddr'.
 * --------------------------------------------------------------------------*/
uint8_t mcp_read_byte(uint8_t chipaddr,uint8_t regaddr,uint8_t *data)
{
  /* put chip address into control bytes */
  chipaddr = chipaddr & 0b00000111;
  if( chipaddr == EEPROM_ADDR )
    return MCP_IO_ERROR; /* do not use this module to talk to eeprom */
  chipaddr = chipaddr << 1; /* move address to b3/b2/b1 */
  ctl_write = MCP_CTL_WRITE | chipaddr;  /* put address into ctl bytes */
  ctl_read  = MCP_CTL_READ  | chipaddr;

  i2c_start();
  
  /* CTL Byte */
  if(!i2c_write_byte(ctl_write))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  /* Address */
  if(!i2c_write_byte(regaddr))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  /* 'restart' */
  i2c_start(); 

  /* CTL Byte */
  if(!i2c_write_byte(ctl_read))
    {
      i2c_stop();
      return MCP_IO_ERROR;
    }
  
  /* Read + ACK */
  *data=i2c_read_byte(0);
  
  i2c_stop();
  
  return MCP_SUCCESS;
}



